## Snapshot

Each layer has its own context. If our scene has multiple layers, and we need to save the current rendering results of the scene, we do not need to process each layer. 

SpriteJS<sup>Next</sup> provides an synchronous interface snapshot (). We can take a current "snapshot" of the scene. `Snapshot()` returns a canvas object, which is the superposition of all the output content of the current layers.(If you want to ignore some "ignoreLayer" content output, you can implement it by setting `ignoreLayer.options.ignoreSnapshot = true `.)

```js
const {Scene, Sprite, Path} = spritejs;
const container = document.getElementById('adaptive');
const scene = new Scene({
  container,
  width: 1200,
  height: 600,
});
const bglayer = scene.layer('bglayer');
const fglayer = scene.layer('fglayer');

(async function () {
  function randomTriangle() {
    const triangle = new Path();
    const pos = [Math.random() * 1540, Math.random() * 600];
    const d = 'M0,0L0,10L8.66, 5z';
    triangle.attr({
      d,
      scale: 5,
      pos,
      fillColor: '#37c',
      rotate: Math.random() * 360,
    });
    bglayer.append(triangle);
  }

  for(let i = 0; i < 100; i++) {
    randomTriangle();
  }

  const birdsJsonUrl = 'https://s5.ssl.qhres2.com/static/5f6911b7b91c88da.json';
  const birdsRes = 'https://p.ssl.qhimg.com/d/inn/c886d09f/birds.png';

  await scene.preload([birdsRes, birdsJsonUrl]);

  const bird = new Sprite('bird1.png');

  bird.attr({
    anchor: [0.5, 0.5],
    pos: [50, 200],
    scale: 0.6,
    offsetPath: 'M0,100C0,100,200,-100,500,100S700,-100,1000,100S1200,-100,1700,100S2200,-100,2700,100',
  });
  fglayer.append(bird);

  bird.animate([
    {texture: 'bird1.png'},
    {texture: 'bird2.png'},
    {texture: 'bird3.png'},
    {texture: 'bird1.png'},
  ], {
    duration: 300,
    iterations: Infinity,
    direction: 'alternate',
    easing: 'step-end',
  });

  bird.animate([
    {offsetDistance: 0},
    {offsetDistance: 1},
  ], {
    duration: 6000,
    iterations: Infinity,
  });

  const snapshotBtn = document.getElementById('take-snapshot'),
    snapshotList = document.getElementById('snapshot-list');

  snapshotBtn.addEventListener('click', async (evt) => {
    const canvas = await scene.snapshot();
    const snapshot = new Image();
    snapshot.src = canvas.toDataURL();
    const listItem = document.createElement('li');
    listItem.appendChild(snapshot);
    snapshotList.appendChild(listItem);
  });
}());
```

<iframe src="/demo/#/doc/snapshot" height="500"></iframe>

## Offscreen Snapshot

When calling scene's snapshot method, if the parameter `{offscreen: true}` is set, the snapshot will return an Offscreen Canvas.